#include "mex.h"
#include <math.h>
#include "bitstream.h"
#include "bitplane.h"
#include "interleave.h"
#include "decoder.h"

void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[]) {

	/* Declarations */
	char *filename;
	int filename_len;
	FILE *fid;
	double bitrate;
	int maxBlock,shift;
	decoder *dec;
	int numSamples,numSlots,numBlocks;
	int output_len;
	double *output;
	int n,numFrames,numFreqs,bin;
	int block,frame,freq;
	double amp;
	mxArray *p,*temp;

	/* Check number of arguments */
	if (nrhs<2) mexErrMsgTxt( "Not enough input arguments.");
	if (nlhs>1) mexErrMsgTxt( "Too many output arguments.");

	/* Input */
	filename_len = mxGetN(prhs[0])+1;
	filename = (char*)mxCalloc(filename_len, sizeof(char));
	mxGetString(prhs[0],filename,filename_len);
	fid = fopen(filename,"r");
	if (fid==NULL) mexErrMsgTxt( "Can't open file.");
	bitrate = mxGetPr(prhs[1])[0];
	if (bitrate<=0||bitrate>1024000) mexErrMsgTxt( "Bitrate must be in the range [0,1024000].");

	/* Parameters */
	if (nrhs>=3) {
		if (!mxIsStruct(prhs[2])) mexErrMsgTxt("Parameters must be a structure.");
		p = (mxArray*)prhs[2];
	} else {
		p = mxCreateStructMatrix(0,0,0,NULL);
	}
	maxBlock = 2;
	temp = mxGetField(p, 0, "maxBlock");
	if (temp!=NULL) maxBlock = (int)mxGetPr(temp)[0];
	shift = 512;
	temp = mxGetField(p, 0, "hopsize");
	if (temp!=NULL) shift = (int)mxGetPr(temp)[0];

	/* Init decoder */
	dec = new decoder( fid );
	if (dec->sampleRate!=44100) {
		delete( dec );
		mexErrMsgTxt( "Invalid file.");
	}
	numSamples = dec->numSamples;
	numSlots = dec->numSlots;
	numBlocks = dec->numBlocks;

	/* Init output */
	if (nlhs>0) {
		output_len = (int)ceil((double)(numSamples-2*32768)/(double)shift);
		plhs[0] = mxCreateDoubleMatrix(output_len,1,mxREAL);
		output = mxGetPr(plhs[0]);
	}

	/* Decoding */
	for ( n=0 ; n<numSlots ; n++ ) {
		if (dec->decode_slot(bitrate)) {
			for ( block=0 ; block<maxBlock ; block++ ) {
				numFrames = dec->get_numFrames(block);
				numFreqs = dec->get_numFreqs(block);
				for ( frame=0 ; frame<numFrames ; frame++ ) {
					bin = ((n*numFrames+frame)*numFreqs-32768+8192) / shift;
					if (bin>=0&&bin<output_len) {
						for ( freq=0 ; freq<numFreqs ; freq++ ) {
							amp = dec->get_coef(block,frame,freq);
							if (amp<0) amp = -amp;
							if (nlhs>0) output[bin] += amp;
						}
					}
				}
			}

		}
	}

	/* Clean */
	delete( dec );
	fclose( fid );
}
