%CODMFCCMEAN Compute transform-domain MFCC+mean and var in texture windows
%   C=CODMFCCMEAN(X,BR,NC,SKIPBEG,SKIPEND,N) returns mean and variance in
%   texture windows of the MFCC of the signal X with bitrate BR and number 
%   of coefficients given by the vector NC. The functions skips the first
%   SKIPBEG and the last SKIPEND frames. Mean and variance are computed in
%   texture windows of size N frames.

function out = codmfccmean(x,bitrate,numcoefs,skipbegin,skipend,framelen)

% Compute MFCC
c = codmfcc(x,bitrate,numcoefs);
c = c';

% Number of frames
numframes = floor((size(c,2)-skipbegin-skipend)/framelen);

%Init output
numcoefs = size(c,1);
out = zeros(numcoefs*2,numframes);

%Compute Mean and Var
for n=1:numframes
    
    idx = (n-1)*framelen+skipbegin+1:(n-1)*framelen+skipbegin+framelen;
    
    out(1:numcoefs,n) = mean(c(:,idx),2);
    out(numcoefs+1:2*numcoefs,n) = var(c(:,idx),0,2);
    
end
