#include "mex.h"
#include <math.h>
#include "bitstream.h"
#include "bitplane.h"
#include "interleave.h"
#include "decoder.h"
#include "imdct.h"

void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[]) {

	/* Declarations */
	char *filename;
	int filename_len;
	FILE *fid;
	double bitrate;
	double *output;
	decoder *dec;
	unsigned int numSamples,sampleRate,numSlots,n;
	int block,frame,freq,pos,sample,output_len;
	int numBlocks,numFrames,numFreqs;
	imdct *im[8];
	double buffer_in[16384];
	double buffer_out[16384];
	double scale = 1/sqrt(2);

	/* Check number of arguments */
	if (nrhs<2) mexErrMsgTxt( "Not enough input arguments.");
	if (nlhs>2) mexErrMsgTxt( "Too many output arguments.");

	/* Input */
	filename_len = mxGetN(prhs[0])+1;
	filename = (char*)mxCalloc(filename_len, sizeof(char));
	mxGetString(prhs[0],filename,filename_len);
	fid = fopen(filename,"r");
	if (fid==NULL) mexErrMsgTxt( "Can't open file.");
	bitrate = mxGetPr(prhs[1])[0];
	if (bitrate<=0||bitrate>1024000) mexErrMsgTxt( "Bitrate must be in the range [0,1024000].");

	/* Init decoder */
	dec = new decoder( fid );
	numSamples = dec->numSamples;
	sampleRate = dec->sampleRate;
	if (sampleRate!=44100) {
		delete( dec );
		mexErrMsgTxt( "Invalid file.");
	}
	numSlots = dec->numSlots;
	numBlocks = dec->numBlocks;
	for ( block=0 ; block<numBlocks ; block++ ) im[block] = new imdct( 128 << block );

	/* Init output */
	if (nlhs>0) {
		output_len = numSamples-2*32768;
		plhs[0] = mxCreateDoubleMatrix(output_len,1,mxREAL);
		output = mxGetPr(plhs[0]);
	}

	/* Decoding */
	for ( n=0 ; n<numSlots ; n++ ) {
		if (dec->decode_slot(bitrate)) {
			for ( block=0 ; block<numBlocks ; block++ ) {
				numFrames = dec->get_numFrames(block);
				numFreqs = dec->get_numFreqs(block);
				for ( frame=0 ; frame<numFrames ; frame++ ) {
					for ( freq=0 ; freq<numFreqs ; freq++ ) {
						buffer_in[freq] = dec->get_coef(block,frame,freq)*scale;
					}
					im[block]->compute(buffer_in,buffer_out);
					if (nlhs>0) {
						pos = n*8192 + frame*numFreqs - (numFreqs>>1) - 24576;
						for ( sample=0 ; sample<(numFreqs<<1) ; sample++ ) {
							if (pos+sample>=0&&pos+sample<output_len) {
								output[pos+sample] += buffer_out[sample];
							}
						}
					}
				}
			}

		}
	}

	/* Output sample rate if needed */
	if (nlhs>=2) {
		plhs[1] = mxCreateDoubleMatrix(1,1,mxREAL);
		mxGetPr(plhs[1])[0] = (double)sampleRate;
	}

	/* Clean */
	delete( dec );
	for ( block=0 ; block<numBlocks ; block++ ) delete( im[block] );
	fclose( fid );
}
