/***************************************************************************
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <math.h>
#include "decoder.h"

decoder::decoder( FILE *set_fid ) {

	/* Input file */
	fid = set_fid;

	/* Read header */
	fread(&numSamples,sizeof(unsigned int),1,fid);
	fread(&sampleRate,sizeof(unsigned int),1,fid);
	fread(&firstNonZeroSlot,sizeof(unsigned int),1,fid);
	fread(&lastNonZeroSlot,sizeof(unsigned int),1,fid);
	fread(&is_psy,sizeof(bool),1,fid);

	/* Init objects */
	bs = new bitstream( 65536 );
	bp = new bitplane( 65536 );
	ci = new interleave(8192,8);

	/* Init output buffer */
	buffer = new double[65536];

	/* Init other variables */
	numSlots = numSamples/8192;
	numBlocks = 8;
	slot = 0;
}



decoder::~decoder( void ) {

	/* Clean */
	delete( bs );
	delete( bp );
	delete( ci );
	delete[]( buffer );
}

int decoder::decode_slot( double bitrate ) {

	unsigned int numBytes,maxNumBytes,numBytes_dec;

	if ( slot>=firstNonZeroSlot && slot<=lastNonZeroSlot ) {

		/* Target number of bytes */
		if (bitrate<=0||bitrate>1024000) printf("Bitrate out of range\n");
		numBytes = (unsigned int)( (8192.0 * bitrate) / ((double)8*(double)sampleRate) ) + 1;

		/* Read data */
		fread(&maxNumBytes,sizeof(unsigned int),1,fid);
		numBytes_dec = numBytes;
		if (maxNumBytes<numBytes) numBytes_dec = maxNumBytes;
		fread(bs->bytes,sizeof(unsigned char),numBytes_dec,fid);
		fseek(fid,maxNumBytes-numBytes_dec,SEEK_CUR);

		/* Decode */
		bs->reset();
		bs->set_maxNumBits( numBytes_dec*8 );
		bp->decode( bs, buffer, sqrt(16384.0), 16, 2 );

		slot++;
		return 1;
	} else {

		slot++;
		return 0;
	}
}

int decoder::get_numFrames( int block ) {

	return 128 >> block;
}

int decoder::get_numFreqs( int block ) {

	return 64 << block;
}

double decoder::get_coef( int block, int frame, int freq ) {

	return buffer[ci->inter[block][frame][freq]];
}

