/***************************************************************************
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <math.h>
#include "kiss_fft.h"
#include "imdct.h"


imdct::imdct( unsigned int set_len ) {

	/* Set parameters */
	len = set_len;
	len2 = len >> 1;
	len4 = len2 >> 1;
	len8 = len4 >> 1;

	/* Init fft */
	infft = new kiss_fft_cpx[len4];
	outfft = new kiss_fft_cpx[len4];
	cfg = kiss_fft_alloc( len4 , 1, 0, 0 );

	/* Init arrays */
	twidRe = new double[len4];
	twidIm = new double[len4];
	window = new double[len];

	/* Init transform */
	unsigned int i;
	double c = sqrt(2.0) / sqrt(sqrt((double)len));
	for ( i = 0 ; i < len4 ; i++ ) {
		twidRe[i] = c * cos( 2.0*M_PI * (8.0*(double)i+1.0) / (8.0*(double)len) );
		twidIm[i] = c * sin( 2.0*M_PI * (8.0*(double)i+1.0) / (8.0*(double)len) );
	}

	/* Init window */
	for ( i=0 ; i<len ; i++ ) window[i] = sin(M_PI*(i+0.5)/len);
}

imdct::~imdct( void ) {

	/* Clean memory */
	free( cfg );
	delete[]( infft );
	delete[]( outfft );
	delete[]( twidRe );
	delete[]( twidIm );
	delete[]( window );
}

void imdct::compute( double *in, double *out ) {

	unsigned int i,k;
	double tempRe;
	double tempIm;

	/* Pre-twiddle */
	for ( i = 0 ; i < len4 ; i++ ) {
		k = i << 1;

		infft[i].r = in[len2-1-k] * twidRe[i] - in[k] * twidIm[i];
		infft[i].i = in[len2-1-k] * twidIm[i] + in[k] * twidRe[i];
	}

	/* Compute fast fourier transform */
	kiss_fft( cfg , infft , outfft );

	/* Post-twiddle */
	for ( i=0 ; i<len4 ; i++ ) {
		tempRe = outfft[i].r;
		tempIm = outfft[i].i;
		outfft[i].r = tempRe * twidRe[i] - tempIm * twidIm[i];
		outfft[i].i = tempRe * twidIm[i] + tempIm * twidRe[i];
	}

	/* Output */
	for ( i = 0 ; i < len8 ; i++ ) {
		k = i << 1;

		out[k] =  outfft[len8+i].i * window[k];
		out[1+k] = - outfft[len8-1-i].r * window[1+k];
		out[len4+k] =  outfft[i].r * window[len4+k];
		out[len4+1 + k] = - outfft[len4-1-i].i * window[len4+1 + k];
		out[len2+k] =  outfft[len8+i].r * window[len2+k];
		out[len2+1+k] = - outfft[len8-1-i].i * window[len2+1+k];
		out[len2+len4+k] = - outfft[i].i * window[len2+len4+k];
		out[len2+len4+1+k] =  outfft[len4-1-i].r * window[len2+len4+1+k];
	}
}

