/***************************************************************************
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <math.h>
#include "interleave.h"

interleave::interleave(unsigned int set_block_len, unsigned int set_numBlocks) {

	/* Declarations */
	unsigned int l,m,n,block;

	/* Set parameters */
	block_len = set_block_len;
	numBlocks = set_numBlocks;

	/* Init arrays */
	numFrames = new unsigned int[numBlocks];
	numFreqs = new unsigned int[numBlocks];
	inter = new unsigned int**[numBlocks];
	for ( l=0 ; l<numBlocks ; l++ ) {
		numFrames[l] = 1 << (numBlocks-1-l);
		numFreqs[l] = block_len >> (numBlocks-1-l);
		inter[l] = new unsigned int*[numFrames[l]];
		for ( n=0 ; n<numFrames[l] ; n++ ) {
			inter[l][n] = new unsigned int[numFreqs[l]];
		}
	}

	/* Init interleaving matrix */
	unsigned int framePermutOld[numFreqs[numBlocks-1]/numFreqs[0]];
	unsigned int framePermutNew[numFreqs[numBlocks-1]/numFreqs[0]];
	for ( n=0 ; n<numBlocks ; n++ ) {
		block = numBlocks-n-1;
		if (block==numBlocks-1) {
			framePermutOld[0] = 0;
		} else {
			for ( m=0 ; m<numFrames[block+1] ; m++ ) {
				framePermutNew[2*m] = framePermutOld[m];
				framePermutNew[2*m+1] = framePermutOld[m] + numFrames[block+1];
			}
			for ( m=0 ; m<numFrames[block] ; m++ ) {
				framePermutOld[m] = framePermutNew[m];
			}
		}
		for ( l=0 ; l<numFreqs[block] ; l++ ) {
			for ( m=0 ; m<numFrames[block] ; m++ ) {
				inter[block][framePermutOld[m]][l] = (l*numFrames[block]+m)*numBlocks+n;
			}
		}
	}
}

interleave::~interleave( void ) {

	/* Declarations */
	unsigned int l,n;

	/* Delete */
	for ( l=0 ; l<numBlocks ; l++ ) {
		for ( n=0 ; n<numFrames[l] ; n++ ) {
			delete[] inter[l][n];
		}
		delete[] inter[l];
	}
	delete[]( numFrames );
	delete[]( numFreqs );
	delete[]( inter );
}

