clear all
close all

%Parameters
low = 20;
high = 16000;
Ns = 2.^(7:14);
nf = length(Ns);
M = 40;
Fs = 44100;
filename = 'filters.h';

%Filters parameters
lowestMel = log(1.0+low/700.0)*1127.01048;
highestMel = log(1.0+high/700.0)*1127.01048;
freqs = zeros(1,M+2);
for n=1:M+2
    freqs(n) = 700.0*( exp( (lowestMel+((n-1)/(M+1))*(highestMel-lowestMel)) / 1127.01048 ) - 1 );
end
lower = freqs(1:end-2);
center = freqs(2:end-1);
upper = freqs(3:end);
heights = 2./(upper-lower);

%Filters
filters = cell(1,nf);
first = cell(1,nf);
last = cell(1,nf);
diff_max = 0;
for f=1:nf
    N = Ns(f);
    fftfs = ((0:N/2-1)+0.5)*Fs/N;
    filters{f} = zeros(M,N/2);
    first{f} = zeros(M,1);
    last{f} = zeros(M,1);
    for n=1:M
        for m=1:N/2
            if fftfs(m)>lower(n)&&fftfs(m)<=center(n)
                filters{f}(n,m) = heights(n) * (fftfs(m)-lower(n))/(center(n)-lower(n));
            end
            if fftfs(m)>center(n)&&fftfs(m)<=upper(n)
                filters{f}(n,m) = heights(n) * (upper(n)-fftfs(m))/(upper(n)-center(n));
            end
        end
        temp = find(filters{f}(n,:));
        if ~isempty(temp)
            first{f}(n) = temp(1)-1;
            last{f}(n) = temp(end)-1;
        end
        if last{f}(n)-first{f}(n)>=diff_max
            diff_max = last{f}(n)-first{f}(n);
        end
    end
end

%Write to file
fid=fopen(filename,'w');

fprintf(fid,'const int filters_first[%d][%d]={',nf,M);
for f=1:nf
    fprintf(fid,'{');
    for n=1:length(first{f})-1
        fprintf(fid,'%d',first{f}(n));
        fprintf(fid,',');
    end
    fprintf(fid,'%d',first{f}(end));
    fprintf(fid,'}');
    if f<nf
        fprintf(fid,',');
    end
end
fprintf(fid,'};\n\n');

fprintf(fid,'const int filters_last[%d][%d]={',nf,M);
for f=1:nf
    fprintf(fid,'{');
    for n=1:length(last{f})-1
        fprintf(fid,'%d',last{f}(n));
        fprintf(fid,',');
    end
    fprintf(fid,'%d',last{f}(end));
    fprintf(fid,'}');
    if f<nf
        fprintf(fid,',');
    end
end
fprintf(fid,'};\n\n');

fprintf(fid,'const double filters_weight[%d][%d][%d]={',nf,M,diff_max+1);
for f=1:nf
    fprintf(fid,'{');
    for n=1:M
        fprintf(fid,'{');
        for m=first{f}(n)+1:last{f}(n)+1
            fprintf(fid,'%.20f',filters{f}(n,m));
            if n<M||m<Ns(f)/2
                fprintf(fid,',');
            end
        end
        fprintf(fid,'}');
        if n<M
            fprintf(fid,',');
        end
    end
    fprintf(fid,'}');
    if f<nf
        fprintf(fid,',');
    end
end
fprintf(fid,'};\n\n');
fclose(fid);